# SPDX-FileCopyrightText: 2026 Mitogen authors <https://github.com/mitogen-hq>
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import absolute_import, division, print_function
__metaclass__ = type

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.source_control import bitbucket


def main():
    module = AnsibleModule(argument_spec={})
    module.exit_json()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# SPDX-FileCopyrightText: 2026 Mitogen authors <https://github.com/mitogen-hq>
# SPDX-License-Identifier: BSD-3-Clause
from __future__ import absolute_import, division, print_function

import ci_lib

if ci_lib.SKIP_CONTAINER_TESTS:
    raise SystemExit(0)

containers = ci_lib.container_specs(ci_lib.DISTRO_SPECS.split())
ci_lib.pull_container_images(containers)



"""
Measure latency of SSH RPC.
"""

import getpass

import mitogen.core
import mitogen.utils
import ansible_mitogen.affinity

mitogen.utils.setup_gil()
ansible_mitogen.affinity.policy.assign_worker()


def do_nothing():
    pass

@mitogen.main()
def main(router):
    import optparse
    parser = optparse.OptionParser(description=__doc__)
    parser.add_option(
        '-p', '--python', metavar='CMD', default='python3',
        help='Remote python path (default %default)')
    parser.add_option(
        '-u', '--user', metavar='S', default=getpass.getuser(),
        help='Remote username (default %default)')
    parser.add_option('--debug', action='store_true')
    opts, args = parser.parse_args()

    f = router.ssh(
        hostname=args[0], python_path=opts.python, username=opts.user,
        debug=opts.debug,
    )
    f.call(do_nothing)
    t0 = mitogen.core.now()
    end = mitogen.core.now() + 5.0
    i = 0
    while mitogen.core.now() < end:
        f.call(do_nothing)
        i += 1

    t1 = mitogen.core.now()
    iterations = i + 1
    mean = (t1 - t0) / iterations
    print('++ iterations %d, mean %.03f ms' % (iterations, 1e3 * mean))



"""
Measure latency of local RPC.
"""

import mitogen.core
import mitogen.utils
import ansible_mitogen.affinity

mitogen.utils.setup_gil()
ansible_mitogen.affinity.policy.assign_worker()


def do_nothing():
    pass

@mitogen.main()
def main(router):
    import optparse
    parser = optparse.OptionParser(description=__doc__)
    parser.add_option(
        '-i', '--iterations', type=int, metavar='N', default=20000,
        help='Number of iterations (default %default)')
    parser.add_option('--debug', action='store_true')
    opts, args = parser.parse_args()

    f = router.fork(debug=opts.debug)
    f.call(do_nothing)
    t0 = mitogen.core.now()
    for x in mitogen.core.range(opts.iterations):
        f.call(do_nothing)

    t1 = mitogen.core.now()
    mean = (t1 - t0) / opts.iterations
    print('++ iterations %d, mean %.03f us' % (opts.iterations, 1e6 * mean))



import re
import sys

sys.path.append('.')


def changelog_version(path, encoding='utf-8'):
    "Return the 1st *stable* (not pre, dev) version in the changelog"
    # See also grep_version() in setup.py
    # e.g. "0.1.2, (1999-12-31)\n"
    version_pattern = re.compile(
        r'^v(?P<version>\d+\.\d+\.\d+) \((?P<date>\d\d\d\d-\d\d-\d\d)\)$',
        re.MULTILINE,
    )

    with open(path, encoding=encoding) as f:
        match = version_pattern.search(f.read())
        return match.group('version')


VERSION = changelog_version('changelog.rst')

author = u'Network Genomics'
copyright = u'2021, the Mitogen authors'
exclude_patterns = ['_build', '.venv']
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.intersphinx', 'sphinxcontrib.programoutput', 'domainrefs']

# get rid of version from <title>, it messes with piwik
html_title = 'Mitogen Documentation'

html_show_copyright = False
html_show_sourcelink = False
html_show_sphinx = False
html_sidebars = {'**': ['globaltoc.html', 'github.html']}
html_static_path = ['_static']
html_theme = 'alabaster'
html_theme_options = {
    'font_family': "Georgia, serif",
    'head_font_family': "Georgia, serif",
    'fixed_sidebar': True,
    'show_powered_by': False,
    'pink_2': 'fffafaf',
    'pink_1': '#fff0f0',
}
htmlhelp_basename = 'mitogendoc'
intersphinx_mapping = {'python': ('https://docs.python.org/3', None)}
language = None
master_doc = 'toc'
project = u'Mitogen'
pygments_style = 'sphinx'
release = VERSION
source_suffix = '.rst'
templates_path = ['_templates']
todo_include_todos = False
version = VERSION

domainrefs = {
    'gh:commit': {
        'text': '%s',
        'url': 'https://github.com/mitogen-hq/mitogen/commit/%s',
    },
    'gh:issue': {
        'text': '#%s',
        'url': 'https://github.com/mitogen-hq/mitogen/issues/%s',
    },
    'gh:pull': {
        'text': '#%s',
        'url': 'https://github.com/mitogen-hq/mitogen/pull/%s',
    },
    'gh:ansissue': {
        'text': 'Ansible #%s',
        'url': 'https://github.com/ansible/ansible/issues/%s',
    },
    'gh:anspull': {
        'text': 'Ansible #%s',
        'url': 'https://github.com/ansible/ansible/pull/%s',
    },

    'ans:mod': {
        'text': '%s module',
        'url': 'https://docs.ansible.com/ansible/latest/modules/%s_module.html',
    },
    'ans:conn': {
        'text': '%s connection plug-in',
        'url': 'https://docs.ansible.com/ansible/latest/plugins/connection/%s.html',
    },
    'freebsd:man2': {
        'text': '%s(2)',
        'url': 'https://man.freebsd.org/cgi/man.cgi?query=%s',
    },
    'linux:man1': {
        'text': '%s(1)',
        'url': 'https://man7.org/linux/man-pages/man1/%s.1.html',
    },
    'linux:man2': {
        'text': '%s(2)',
        'url': 'https://man7.org/linux/man-pages/man2/%s.2.html',
    },
    'linux:man3': {
        'text': '%s(3)',
        'url': 'https://man7.org/linux/man-pages/man3/%s.3.html',
    },
    'linux:man7': {
        'text': '%s(7)',
        'url': 'https://man7.org/linux/man-pages/man7/%s.7.html',
    },
}

# > ## Official guidance
# > Query PyPI’s JSON API to determine where to download files from.
# > ## Predictable URLs
# > You can use our conveyor service to fetch this file, which exists for
# > cases where using the API is impractical or impossible.
# > -- https://warehouse.pypa.io/api-reference/integration-guide.html#predictable-urls
rst_epilog = """

.. |mitogen_version| replace:: %(VERSION)s

.. |mitogen_url| replace:: `mitogen-%(VERSION)s.tar.gz <https://files.pythonhosted.org/packages/source/m/mitogen/mitogen-%(VERSION)s.tar.gz>`__

""" % locals()



#!/usr/bin/env python
# Run tests/ansible/all.yml under Ansible and Ansible-Mitogen

import collections
import glob
import os
import sys

import jinja2

import ci_lib


TMP = ci_lib.TempDir(prefix='mitogen_ci_ansible')
TMP_HOSTS_DIR = os.path.join(TMP.path, 'hosts')

interesting = ci_lib.get_interesting_procs()

with ci_lib.Fold('docker_setup'):
    containers = ci_lib.container_specs(ci_lib.DISTRO_SPECS.split())
    ci_lib.start_containers(containers)


with ci_lib.Fold('job_setup'):
    os.chdir(ci_lib.ANSIBLE_TESTS_DIR)

    os.mkdir(TMP_HOSTS_DIR)
    for path in glob.glob(os.path.join(ci_lib.ANSIBLE_TESTS_HOSTS_DIR, '*')):
        if not path.endswith('default.hosts'):
            os.symlink(path, os.path.join(TMP_HOSTS_DIR, os.path.basename(path)))

    distros = collections.defaultdict(list)
    families = collections.defaultdict(list)
    for container in containers:
        distros[container['distro']].append(container['name'])
        families[container['family']].append(container['name'])

    jinja_env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(
            searchpath=ci_lib.ANSIBLE_TESTS_TEMPLATES_DIR,
        ),
        lstrip_blocks=True,  # Remove spaces and tabs from before a block
        trim_blocks=True,  # Remove first newline after a block
    )
    inventory_template = jinja_env.get_template('test-targets.j2')
    inventory_path = os.path.join(TMP_HOSTS_DIR, 'test-targets.ini')

    with open(inventory_path, 'w') as fp:
        fp.write(inventory_template.render(
            containers=containers,
            distros=distros,
            families=families,
        ))

    ci_lib.dump_file(inventory_path)

with ci_lib.Fold('ansible'):
    playbook = os.environ.get('PLAYBOOK', 'all.yml')
    ci_lib.run(
        './run_ansible_playbook.py %s -i "%s" %s',
        playbook, TMP_HOSTS_DIR, ' '.join(sys.argv[1:]),
    )

ci_lib.check_stray_processes(interesting, containers)



from __future__ import absolute_import
import io
import os
import stat
import sys
import tempfile
import unittest

if sys.version_info >= (3,):
    import configparser
    from unittest import mock
else:
    import ConfigParser as configparser
    import mock

import ansible.errors
import ansible.playbook.play_context

import mitogen.core

import ansible_mitogen.connection
import ansible_mitogen.plugins.connection.mitogen_local
import ansible_mitogen.process


def base_executable(executable=None):
    '''Return the path of the Python executable used to create the virtualenv.
    '''
    # https://docs.python.org/3/library/venv.html
    # https://github.com/pypa/virtualenv/blob/main/src/virtualenv/discovery/py_info.py
    # https://virtualenv.pypa.io/en/16.7.9/reference.html#compatibility-with-the-stdlib-venv-module
    if executable is None:
        executable = sys.executable

    if not executable:
        raise ValueError

    try:
        base_executable = sys._base_executable
    except AttributeError:
        base_executable = None

    if base_executable and base_executable != executable:
        return base_executable

    # Python 2.x only has sys.base_prefix if running outside a virtualenv.
    try:
        sys.base_prefix
    except AttributeError:
        # Python 2.x outside a virtualenv
        return executable

    # Python 3.3+ has sys.base_prefix. In a virtualenv it differs to sys.prefix.
    if sys.base_prefix == sys.prefix:
        return executable

    while executable.startswith(sys.prefix) and stat.S_ISLNK(os.lstat(executable).st_mode):
        dirname = os.path.dirname(executable)
        target = os.path.join(dirname, os.readlink(executable))
        executable = os.path.abspath(os.path.normpath(target))
        print(executable)

    if executable.startswith(sys.base_prefix):
        return executable

    # Virtualenvs record details in pyvenv.cfg
    parser = configparser.RawConfigParser()
    with io.open(os.path.join(sys.prefix, 'pyvenv.cfg'), encoding='utf-8') as f:
        content = u'[virtualenv]\n' + f.read()
    try:
        parser.read_string(content)
    except AttributeError:
        parser.readfp(io.StringIO(content))

    # virtualenv style pyvenv.cfg includes the base executable.
    # venv style pyvenv.cfg doesn't.
    try:
        return parser.get(u'virtualenv', u'base-executable')
    except configparser.NoOptionError:
        pass

    basename = os.path.basename(executable)
    home = parser.get(u'virtualenv', u'home')
    return os.path.join(home, basename)


class MuxProcessMixin(object):
    no_zombie_check = True

    @classmethod
    def setUpClass(cls):
        cls.model = ansible_mitogen.process.get_classic_worker_model(
            _init_logging=False
        )
        ansible_mitogen.process.set_worker_model(cls.model)
        cls.model.on_strategy_start()
        super(MuxProcessMixin, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        cls.model._test_reset()
        super(MuxProcessMixin, cls).tearDownClass()


class ConnectionMixin(MuxProcessMixin):
    klass = ansible_mitogen.plugins.connection.mitogen_local.Connection

    def make_connection(self):
        play_context = ansible.playbook.play_context.PlayContext()
        conn = self.klass(play_context, new_stdin=False)
        # conn functions don't fetch ActionModuleMixin objs from _get_task_vars()
        # through the usual walk-the-stack approach so we'll not run interpreter discovery here
        conn._action = mock.MagicMock(
            _mitogen_interpreter_candidate=base_executable(),
        )
        conn.on_action_run(
            task_vars={},
            delegate_to_hostname=None,
            loader_basedir=None,
        )

        return conn

    def wait_for_completion(self):
        # put_data() is asynchronous, must wait for operation to happen. Do
        # that by making RPC for some junk that must run on the thread once op
        # completes.
        self.conn.get_chain().call(os.getpid)

    def setUp(self):
        super(ConnectionMixin, self).setUp()
        self.conn = self.make_connection()

    def tearDown(self):
        self.conn.close()
        super(ConnectionMixin, self).tearDown()


class MuxShutdownTest(ConnectionMixin, unittest.TestCase):
    def test_connection_failure_raised(self):
        # ensure if a WorkerProcess tries to connect to a MuxProcess that has
        # already shut down, it fails with a graceful error.
        path = self.model._muxes[0].path
        os.rename(path, path + '.tmp')
        try:
            #e = self.assertRaises(ansible.errors.AnsibleError,
                #lambda: self.conn._connect()
            #)
            e = 1
            print(e)
        finally:
            os.rename(path + '.tmp', path)


class OptionalIntTest(unittest.TestCase):
    func = staticmethod(ansible_mitogen.connection.optional_int)

    def test_already_int(self):
        self.assertEqual(0, self.func(0))
        self.assertEqual(1, self.func(1))
        self.assertEqual(-1, self.func(-1))

    def test_is_string(self):
        self.assertEqual(0, self.func("0"))
        self.assertEqual(1, self.func("1"))
        self.assertEqual(-1, self.func("-1"))

    def test_is_none(self):
        self.assertEqual(None, self.func(None))

    def test_is_junk(self):
        self.assertEqual(None, self.func({1:2}))


class FetchFileTest(ConnectionMixin, unittest.TestCase):
    def test_success(self):
        with tempfile.NamedTemporaryFile(prefix='mitotest') as ifp:
            with tempfile.NamedTemporaryFile(prefix='mitotest') as ofp:
                ifp.write(b'x' * (1048576 * 4))
                ifp.flush()
                ifp.seek(0)

                self.conn.fetch_file(ifp.name, ofp.name)
                # transfer_file() uses os.rename rather than direct data
                # overwrite, so we must reopen.
                with open(ofp.name, 'rb') as fp:
                    self.assertEqual(ifp.read(), fp.read())


class PutDataTest(ConnectionMixin, unittest.TestCase):
    def test_out_path(self):
        path = tempfile.mktemp(prefix='mitotest')
        contents = mitogen.core.b('contents')

        self.conn.put_data(path, contents)
        self.wait_for_completion()
        with open(path, 'rb') as f:
            self.assertEqual(contents, f.read())
        os.unlink(path)

    def test_mode(self):
        path = tempfile.mktemp(prefix='mitotest')
        contents = mitogen.core.b('contents')

        self.conn.put_data(path, contents, mode=int('0123', 8))
        self.wait_for_completion()
        st = os.stat(path)
        self.assertEqual(int('0123', 8), st.st_mode & int('0777', 8))
        os.unlink(path)


class PutFileTest(ConnectionMixin, unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        super(PutFileTest, cls).setUpClass()
        cls.big_path = tempfile.mktemp(prefix='mitotestbig')
        fp = open(cls.big_path, 'w')
        try:
            fp.write('x'*1048576)
        finally:
            fp.close()

    @classmethod
    def tearDownClass(cls):
        os.unlink(cls.big_path)
        super(PutFileTest, cls).tearDownClass()

    def test_out_path_tiny(self):
        path = tempfile.mktemp(prefix='mitotest')
        self.conn.put_file(in_path=__file__, out_path=path)
        self.wait_for_completion()
        with open(path, 'rb') as path_f:
            with open(__file__, 'rb') as __file__f:
                self.assertEqual(path_f.read(), __file__f.read())
        os.unlink(path)

    def test_out_path_big(self):
        path = tempfile.mktemp(prefix='mitotest')
        self.conn.put_file(in_path=self.big_path, out_path=path)
        self.wait_for_completion()
        with open(path, 'rb') as path_f:
            with open(self.big_path, 'rb') as big_path_f:
                self.assertEqual(path_f.read(), big_path_f.read())
        #self._compare_times_modes(path, __file__)
        os.unlink(path)

    def test_big_in_path_not_found(self):
        path = tempfile.mktemp(prefix='mitotest')
        self.assertRaises(ansible.errors.AnsibleFileNotFound,
            lambda: self.conn.put_file(in_path='/nonexistent', out_path=path))



import os.path
import sys
import textwrap
import unittest

import ansible_mitogen.module_finder

ANSIBLE_BASE_DIR = os.path.abspath(os.path.join(__file__, '..', '..'))
ANSIBLE_MODULES_DIR = os.path.join(ANSIBLE_BASE_DIR, 'lib', 'modules')
ANSIBLE_MODULE_UTILS_DIR = os.path.join(ANSIBLE_BASE_DIR, 'lib', 'module_utils')


class ScanFromListTest(unittest.TestCase):
    def test_absolute_imports(self):
        source = textwrap.dedent('''\
            from __future__ import absolute_import
            import a; import b.c; from d.e import f; from g import h, i
        ''')
        code = compile(source, '<str>', 'exec')
        self.assertEqual(
            list(ansible_mitogen.module_finder.scan_fromlist(code)),
            [(0, '__future__.absolute_import'), (0, 'a'), (0, 'b.c'), (0, 'd.e.f'), (0, 'g.h'), (0, 'g.i')],
        )


class WalkImportsTest(unittest.TestCase):
    def test_absolute_imports(self):
        source = textwrap.dedent('''\
            from __future__ import absolute_import
            import a; import b; import b.c; from b.d import e, f
        ''')
        code = compile(source, '<str>', 'exec')

        self.assertEqual(
            list(ansible_mitogen.module_finder.walk_imports(code)),
            ['__future__', '__future__.absolute_import', 'a', 'b', 'b', 'b.c', 'b', 'b.d', 'b.d.e', 'b.d.f'],
        )
        self.assertEqual(
            list(ansible_mitogen.module_finder.walk_imports(code, prefix='b')),
            ['b.c', 'b.d', 'b.d.e', 'b.d.f'],
        )


class ScanTest(unittest.TestCase):
    module_name = 'ansible_module_module_finder_test__this_should_not_matter'
    module_path = os.path.join(ANSIBLE_MODULES_DIR, 'module_finder_test.py')
    search_path = (
        'does_not_exist/module_utils',
        ANSIBLE_MODULE_UTILS_DIR,
    )

    @staticmethod
    def relpath(path):
        return os.path.relpath(path, ANSIBLE_MODULE_UTILS_DIR)

    @unittest.skipIf(sys.version_info < (3, 4), 'find spec() unavailable')
    def test_importlib_find_spec(self):
        scan = ansible_mitogen.module_finder._scan_importlib_find_spec
        actual = scan(self.module_name, self.module_path, self.search_path)
        self.assertEqual(
            [(name, self.relpath(path), is_pkg) for name, path, is_pkg in actual],
            [
                ('ansible.module_utils.external1', 'external1.py', False),
                ('ansible.module_utils.external2', 'external2.py', False),
                ('ansible.module_utils.externalpkg', 'externalpkg/__init__.py', True),
                ('ansible.module_utils.externalpkg.extmod', 'externalpkg/extmod.py',False),
            ],
        )

    @unittest.skipIf(sys.version_info >= (3, 4), 'find spec() preferred')
    def test_imp_find_module(self):
        scan = ansible_mitogen.module_finder._scan_imp_find_module
        actual = scan(self.module_name, self.module_path, self.search_path)
        self.assertEqual(
            [(name, self.relpath(path), is_pkg) for name, path, is_pkg in actual],
            [
                ('ansible.module_utils.external1', 'external1.py', False),
                ('ansible.module_utils.external2', 'external2.py', False),
                ('ansible.module_utils.externalpkg', 'externalpkg/__init__.py', True),
                ('ansible.module_utils.externalpkg.extmod', 'externalpkg/extmod.py',False),
            ],
        )
